@@ -14,6 +14,7 @@
   - Martin Barisits, <martin.barisits@cern.ch>, 2014-2015
   - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2018-2019
   - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
+  - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
 
   PY3K COMPATIBLE
 '''
@@ -530,3 +531,76 @@ def list_archive_content(scope, name):
     dids = did.list_archive_content(scope=scope, name=name)
     for d in dids:
         yield api_update_return_dict(d)
+
+
+def add_did_to_followed(scope, name, account, session=None):
+    """
+    Mark a did as followed by the given account
+
+    :param scope: The scope name.
+    :param name: The data identifier name.
+    :param account: The account owner.
+    :param session: The database session in use.
+    """
+    scope = InternalScope(scope)
+    account = InternalAccount(account)
+    return did.add_did_to_followed(scope=scope, name=name, account=account, session=session)
+
+
+def add_dids_to_followed(dids, account, session=None):
+    """
+    Bulk mark datasets as followed
+
+    :param dids: A list of dids.
+    :param account: The account owner.
+    :param session: The database session in use.
+    """
+    account = InternalAccount(account)
+    return did.add_dids_to_followed(dids=dids, account=account, session=session)
+
+
+def get_users_following_did(name, scope, session=None):
+    """
+    Return list of users following a did
+
+    :param scope: The scope name.
+    :param name: The data identifier name.
+    :param session: The database session in use.
+    """
+    scope = InternalScope(scope)
+    return did.get_users_following_did(name=name, scope=scope, session=session)
+
+
+def remove_did_from_followed(scope, name, account, issuer, session=None):
+    """
+    Mark a did as not followed
+
+    :param scope: The scope name.
+    :param name: The data identifier name.
+    :param account: The account owner.
+    :param session: The database session in use.
+    :param issuer: The issuer account
+    """
+    kwargs = {'scope': scope, 'issuer': issuer}
+    if not rucio.api.permission.has_permission(issuer=issuer, action='remove_did_from_followed', kwargs=kwargs):
+        raise rucio.common.exception.AccessDenied('Account %s can not remove data identifiers from followed table' % (issuer))
+
+    scope = InternalScope(scope)
+    account = InternalAccount(account)
+    return did.remove_did_from_followed(scope=scope, name=name, account=account, session=session)
+
+
+def remove_dids_from_followed(dids, account, issuer, session=None):
+    """
+    Bulk mark datasets as not followed
+
+    :param dids: A list of dids.
+    :param account: The account owner.
+    :param session: The database session in use.
+    """
+    kwargs = {'dids': dids, 'issuer': issuer}
+    if not rucio.api.permission.has_permission(issuer=issuer, action='remove_dids_from_followed', kwargs=kwargs):
+        raise rucio.common.exception.AccessDenied('Account %s can not bulk remove data identifiers from followed table' % (issuer))
+
+    account = InternalAccount(account)
+    return did.remove_dids_from_followed(dids=dids, account=account, session=session)
