@@ -9,16 +9,18 @@
 # - Vincent Garonne, <vincent.garonne@cern.ch>, 2016
 # - Cedric Serfon, <cedric.serfon@cern.ch>, 2016-2018
 # - Mario Lassnig, <mario.lassnig@cern.ch>, 2017-2018
-# - Martin Barisits, <martin.barisits@cern.ch>, 2017
+# - Martin Barisits, <martin.barisits@cern.ch>, 2017-2019
 # - Eric Vaandering, <ewv@fnal.gov>, 2018
 # - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
 #
 # PY3K COMPATIBLE
 
-import rucio.core.authentication
 import rucio.core.scope
 from rucio.core.account import has_account_attribute, list_account_attributes
+from rucio.core.identity import exist_identity_account
 from rucio.core.rse import list_rse_attributes
+from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rule import get_rule
 from rucio.db.sqla.constants import IdentityType
 
@@ -62,6 +64,7 @@ def has_permission(issuer, action, kwargs):
             'get_auth_token_user_pass': perm_get_auth_token_user_pass,
             'get_auth_token_gss': perm_get_auth_token_gss,
             'get_auth_token_x509': perm_get_auth_token_x509,
+            'get_auth_token_saml': perm_get_auth_token_saml,
             'add_account_identity': perm_add_account_identity,
             'add_did': perm_add_did,
             'add_dids': perm_add_dids,
@@ -78,8 +81,10 @@ def has_permission(issuer, action, kwargs):
             'get_request_by_did': perm_get_request_by_did,
             'cancel_request': perm_cancel_request,
             'get_next': perm_get_next,
-            'set_account_limit': perm_set_account_limit,
-            'delete_account_limit': perm_delete_account_limit,
+            'set_local_account_limit': perm_set_local_account_limit,
+            'set_global_account_limit': perm_set_global_account_limit,
+            'delete_local_account_limit': perm_delete_local_account_limit,
+            'delete_global_account_limit': perm_delete_global_account_limit,
             'config_sections': perm_config,
             'config_add_section': perm_config,
             'config_has_section': perm_config,
@@ -90,7 +95,7 @@ def has_permission(issuer, action, kwargs):
             'config_set': perm_config,
             'config_remove_section': perm_config,
             'config_remove_option': perm_config,
-            'get_account_usage': perm_get_account_usage,
+            'get_local_account_usage': perm_get_local_account_usage,
             'add_attribute': perm_add_account_attribute,
             'del_attribute': perm_del_account_attribute,
             'list_heartbeats': perm_list_heartbeats,
@@ -100,7 +105,9 @@ def has_permission(issuer, action, kwargs):
             'get_signed_url': perm_get_signed_url,
             'add_bad_pfns': perm_add_bad_pfns,
             'del_account_identity': perm_del_account_identity,
-            'del_identity': perm_del_identity}
+            'del_identity': perm_del_identity,
+            'remove_did_from_followed': perm_remove_did_from_followed,
+            'remove_dids_from_followed': perm_remove_dids_from_followed}
 
     return perm.get(action, perm_default)(issuer=issuer, kwargs=kwargs)
 
@@ -150,6 +157,18 @@ def perm_add_rule(issuer, kwargs):
     :param kwargs: List of arguments for the action.
     :returns: True if account is allowed, otherwise False
     """
+
+    rses = parse_expression(kwargs['rse_expression'])
+    # If all the RSEs matching the expression need approval, the rule cannot be created
+    if not kwargs['ask_approval']:
+        all_rses_need_approval = True
+        for rse in rses:
+            rse_attr = list_rse_attributes(rse_id=rse['id'])
+            if rse_attr.get('requires_approval', False):
+                all_rses_need_approval = False
+        if not all_rses_need_approval:
+            return False
+
     if kwargs['account'] == issuer and not kwargs['locked']:
         return True
     if _is_root(issuer) or has_account_attribute(account=issuer, key='admin'):
@@ -259,8 +278,7 @@ def perm_get_auth_token_user_pass(issuer, kwargs):
     :param kwargs: List of arguments for the action.
     :returns: True if account is allowed, otherwise False
     """
-    if rucio.core.authentication.exist_identity_account(identity=kwargs['username'], type=IdentityType.USERPASS,
-                                                        account=kwargs['account']):
+    if exist_identity_account(identity=kwargs['username'], type=IdentityType.USERPASS, account=kwargs['account']):
         return True
     return False
 
@@ -273,8 +291,7 @@ def perm_get_auth_token_gss(issuer, kwargs):
     :param kwargs: List of arguments for the action.
     :returns: True if account is allowed, otherwise False
     """
-    if rucio.core.authentication.exist_identity_account(identity=kwargs['gsscred'], type=IdentityType.GSS,
-                                                        account=kwargs['account']):
+    if exist_identity_account(identity=kwargs['gsscred'], type=IdentityType.GSS, account=kwargs['account']):
         return True
     return False
 
@@ -287,8 +304,20 @@ def perm_get_auth_token_x509(issuer, kwargs):
     :param kwargs: List of arguments for the action.
     :returns: True if account is allowed, otherwise False
     """
-    if rucio.core.authentication.exist_identity_account(identity=kwargs['dn'], type=IdentityType.X509,
-                                                        account=kwargs['account']):
+    if exist_identity_account(identity=kwargs['dn'], type=IdentityType.X509, account=kwargs['account']):
+        return True
+    return False
+
+
+def perm_get_auth_token_saml(issuer, kwargs):
+    """
+    Checks if a user can request a token with user_pass for an account.
+
+    :param issuer: Account identifier which issues the command.
+    :param kwargs: List of arguments for the action.
+    :returns: True if account is allowed, otherwise False
+    """
+    if exist_identity_account(identity=kwargs['saml_nameid'], type=IdentityType.SAML, account=kwargs['account']):
         return True
     return False
 
@@ -343,10 +372,10 @@ def perm_add_did(issuer, kwargs):
             if rule['account'] != issuer:
                 return False
 
-    return (_is_root(issuer) or
-            has_account_attribute(account=issuer, key='admin') or
-            rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer) or
-            kwargs['scope'] == u'mock')
+    return (_is_root(issuer)
+            or has_account_attribute(account=issuer, key='admin')  # NOQA: W503
+            or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer)  # NOQA: W503
+            or kwargs['scope'] == u'mock')  # NOQA: W503
 
 
 def perm_add_dids(issuer, kwargs):
@@ -375,10 +404,10 @@ def perm_attach_dids(issuer, kwargs):
     :param kwargs: List of arguments for the action.
     :returns: True if account is allowed, otherwise False
     """
-    return (_is_root(issuer) or
-            has_account_attribute(account=issuer, key='admin') or
-            rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer) or
-            kwargs['scope'] == 'mock')
+    return (_is_root(issuer)
+            or has_account_attribute(account=issuer, key='admin')  # NOQA: W503
+            or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer)  # NOQA: W503
+            or kwargs['scope'] == 'mock')  # NOQA: W503
 
 
 def perm_attach_dids_to_dids(issuer, kwargs):
@@ -409,10 +438,10 @@ def perm_create_did_sample(issuer, kwargs):
     :param kwargs: List of arguments for the action.
     :returns: True if account is allowed, otherwise False
     """
-    return issuer == ('root' or
-                      has_account_attribute(account=issuer, key='admin') or
-                      rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer) or
-                      kwargs['scope'] == 'mock')
+    return issuer == ('root'
+                      or has_account_attribute(account=issuer, key='admin')  # NOQA: W503
+                      or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer)  # NOQA: W503
+                      or kwargs['scope'] == 'mock')  # NOQA: W503
 
 
 def perm_del_rule(issuer, kwargs):
@@ -454,6 +483,17 @@ def perm_approve_rule(issuer, kwargs):
     """
     if _is_root(issuer) or has_account_attribute(account=issuer, key='admin'):
         return True
+
+    rule = get_rule(rule_id=kwargs['rule_id'])
+    rses = parse_expression(rule['rse_expression'])
+
+    # Those in rule_approvers can approve the rule
+    for rse in rses:
+        rse_attr = list_rse_attributes(rse_id=rse['id'])
+        rule_approvers = rse_attr.get('rule_approvers', None)
+        if rule_approvers and issuer.external in rule_approvers.split(','):
+            return True
+
     return False
 
 
@@ -516,8 +556,8 @@ def perm_set_metadata(issuer, kwargs):
     :param kwargs: List of arguments for the action.
     :returns: True if account is allowed, otherwise False
     """
-    return (_is_root(issuer) or has_account_attribute(account=issuer, key='admin') or
-            rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer))
+    return (_is_root(issuer) or has_account_attribute(account=issuer, key='admin')
+            or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer))  # NOQA: W503
 
 
 def perm_set_status(issuer, kwargs):
@@ -532,8 +572,8 @@ def perm_set_status(issuer, kwargs):
         if issuer != 'root' and not has_account_attribute(account=issuer, key='admin'):
             return False
 
-    return (_is_root(issuer) or has_account_attribute(account=issuer, key='admin') or
-            rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer))
+    return (_is_root(issuer) or has_account_attribute(account=issuer, key='admin')
+            or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer))  # NOQA: W503
 
 
 def perm_add_protocol(issuer, kwargs):
@@ -722,7 +762,7 @@ def perm_set_rse_limits(issuer, kwargs):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin')
 
 
-def perm_set_account_limit(issuer, kwargs):
+def perm_set_local_account_limit(issuer, kwargs):
     """
     Checks if an account can set an account limit.
 
@@ -739,10 +779,62 @@ def perm_set_account_limit(issuer, kwargs):
             admin_in_country.append(kv['key'].partition('-')[2])
     if admin_in_country and list_rse_attributes(rse_id=kwargs['rse_id']).get('country') in admin_in_country:
         return True
+
+    # Those listed as quota approvers can add to quotas
+    rse_attr = list_rse_attributes(rse_id=kwargs['rse_id'])
+    quota_approvers = rse_attr.get('quota_approvers', None)
+    if quota_approvers and issuer.external in quota_approvers.split(','):
+        return True
+
     return False
 
 
-def perm_delete_account_limit(issuer, kwargs):
+def perm_set_global_account_limit(issuer, kwargs):
+    """
+    Checks if an account can set a global account limit.
+
+    :param account: Account identifier which issues the command.
+    :param kwargs: List of arguments for the action.
+    :returns: True if account is allowed, otherwise False
+    """
+    if _is_root(issuer) or has_account_attribute(account=issuer, key='admin'):
+        return True
+    # Check if user is a country admin
+    admin_in_country = set()
+    for kv in list_account_attributes(account=issuer):
+        if kv['key'].startswith('country-') and kv['value'] == 'admin':
+            admin_in_country.add(kv['key'].partition('-')[2])
+    resolved_rse_countries = {list_rse_attributes(rse_id=rse['rse_id']).get('country')
+                              for rse in parse_expression(kwargs['rse_exp'])}
+    if resolved_rse_countries.issubset(admin_in_country):
+        return True
+    return False
+
+
+def perm_delete_global_account_limit(issuer, kwargs):
+    """
+    Checks if an account can delete a global account limit.
+
+    :param issuer: Account identifier which issues the command.
+    :param kwargs: List of arguments for the action.
+    :returns: True if account is allowed, otherwise False
+    """
+    if _is_root(issuer) or has_account_attribute(account=issuer, key='admin'):
+        return True
+    # Check if user is a country admin
+    admin_in_country = set()
+    for kv in list_account_attributes(account=issuer):
+        if kv['key'].startswith('country-') and kv['value'] == 'admin':
+            admin_in_country.add(kv['key'].partition('-')[2])
+    if admin_in_country:
+        resolved_rse_countries = {list_rse_attributes(rse_id=rse['rse_id']).get('country')
+                                  for rse in parse_expression(kwargs['rse_exp'])}
+        if resolved_rse_countries.issubset(admin_in_country):
+            return True
+    return False
+
+
+def perm_delete_local_account_limit(issuer, kwargs):
     """
     Checks if an account can delete an account limit.
 
@@ -759,6 +851,12 @@ def perm_delete_account_limit(issuer, kwargs):
             admin_in_country.append(kv['key'].partition('-')[2])
     if admin_in_country and list_rse_attributes(rse_id=kwargs['rse_id']).get('country') in admin_in_country:
         return True
+
+    rse_attr = list_rse_attributes(rse_id=kwargs['rse_id'])
+    quota_approvers = rse_attr.get('quota_approvers', None)
+    if quota_approvers and issuer.external in quota_approvers.split(','):
+        return True
+
     return False
 
 
@@ -773,7 +871,7 @@ def perm_config(issuer, kwargs):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin')
 
 
-def perm_get_account_usage(issuer, kwargs):
+def perm_get_local_account_usage(issuer, kwargs):
     """
     Checks if an account can get the account usage of an account.
 
@@ -875,3 +973,32 @@ def perm_add_bad_pfns(issuer, kwargs):
     :returns: True if account is allowed, otherwise False
     """
     return _is_root(issuer)
+
+
+def perm_remove_did_from_followed(issuer, kwargs):
+    """
+    Checks if an account can remove did from followed table.
+
+    :param issuer: Account identifier which issues the command.
+    :param kwargs: List of arguments for the action.
+    :returns: True if account is allowed, otherwise False
+    """
+    return (_is_root(issuer)
+            or has_account_attribute(account=issuer, key='admin')  # NOQA: W503
+            or kwargs['account'] == issuer  # NOQA: W503
+            or kwargs['scope'] == 'mock')  # NOQA: W503
+
+
+def perm_remove_dids_from_followed(issuer, kwargs):
+    """
+    Checks if an account can bulk remove dids from followed table.
+
+    :param issuer: Account identifier which issues the command.
+    :param kwargs: List of arguments for the action.
+    :returns: True if account is allowed, otherwise False
+    """
+    if _is_root(issuer) or has_account_attribute(account=issuer, key='admin'):
+        return True
+    if not kwargs['account'] == issuer:
+        return False
+    return True
