@@ -16,7 +16,7 @@
 # - Vincent Garonne <vgaronne@gmail.com>, 2012-2018
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2015
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2019
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2018
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2018
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2017
 # - Wen Guan <wguan.icedew@gmail.com>, 2015-2016
@@ -24,8 +24,10 @@
 # - Frank Berghaus <frank.berghaus@cern.ch>, 2018
 # - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018-2019
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+# - Gabriele Fronze' <gfronze@cern.ch>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Brandon White <bjwhite@fnal.gov>, 2019
+# - Brandon White <bjwhite@fnal.gov>, 2019-2020
+# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2019
 #
 # PY3K COMPATIBLE
 
@@ -56,6 +58,7 @@ import rucio.core.account_counter
 from rucio.core.rse_counter import add_counter, get_counter
 from rucio.common import exception, utils
 from rucio.common.config import get_lfn2pfn_algorithm_default, config_get
+from rucio.common.utils import CHECKSUM_KEY, is_checksum_valid, GLOBALLY_SUPPORTED_CHECKSUMS
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import RSEType
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
@@ -69,8 +72,7 @@ REGION = make_region().configure('dogpile.cache.memcached',
 
 @transactional_session
 def add_rse(rse, deterministic=True, volatile=False, city=None, region_code=None, country_name=None, continent=None, time_zone=None,
-            ISP=None, staging_area=False, rse_type=RSEType.DISK, longitude=None, latitude=None, ASN=None, availability=7,
-            session=None):
+            ISP=None, staging_area=False, rse_type=RSEType.DISK, longitude=None, latitude=None, ASN=None, availability=7, session=None):
     """
     Add a rse with the given location name.
 
@@ -118,7 +120,7 @@ def add_rse(rse, deterministic=True, volatile=False, city=None, region_code=None
 
 
 @read_session
-def rse_exists(rse, session=None):
+def rse_exists(rse, include_deleted=False, session=None):
     """
     Checks to see if RSE exists.
 
@@ -127,7 +129,7 @@ def rse_exists(rse, session=None):
 
     :returns: True if found, otherwise false.
     """
-    return True if session.query(models.RSE).filter_by(rse=rse, deleted=False).first() else False
+    return True if session.query(models.RSE).filter_by(rse=rse, deleted=include_deleted).first() else False
 
 
 @read_session
@@ -183,6 +185,27 @@ def del_rse(rse_id, session=None):
         pass
 
 
+@transactional_session
+def restore_rse(rse_id, session=None):
+    """
+    Restore a rse with the given rse id.
+
+    :param rse_id: the rse id.
+    :param session: The database session in use.
+    """
+
+    old_rse = None
+    try:
+        old_rse = session.query(models.RSE).filter_by(id=rse_id, deleted=True).one()
+    except sqlalchemy.orm.exc.NoResultFound:
+        raise exception.RSENotFound('RSE with id \'%s\' cannot be found' % rse_id)
+    old_rse.deleted = False
+    old_rse.deleted_at = None
+    old_rse.save(session=session)
+    rse = old_rse.rse
+    add_rse_attribute(rse_id=rse_id, key=rse, value=True, session=session)
+
+
 @read_session
 def rse_is_empty(rse_id, session=None):
     """
@@ -326,19 +349,8 @@ def list_rses(filters={}, session=None):
             else:
                 t = aliased(models.RSEAttrAssociation)
                 query = query.join(t, t.rse_id == models.RSEAttrAssociation.rse_id)
-                query = query.filter(t.key == k)
-
-                # FIXME
-                # ATLAS RSE listing workaround (since booleans are capital 'True'/'False')
-                # remove elif branch after appropriate database fix has been applied
-                # see also db/types.py
-                if isinstance(v, bool):
-                    query = query.filter(or_(t.value == v,
-                                             t.value == 'tmp_atlas_%s' % v,
-                                             t.value == 'tmp_atlas_%s' % 1 if v else 0))
-                else:
-                    query = query.filter(or_(t.value == v,
-                                             t.value == 'tmp_atlas_%s' % v))
+                query = query.filter(t.key == k,
+                                     t.value == v)
 
         condition1, condition2 = [], []
         for i in range(0, 8):
@@ -547,6 +559,47 @@ def get_rse_attribute(key, rse_id=None, value=None, use_cache=True, session=None
     return result
 
 
+@read_session
+def get_rse_supported_checksums(rse_id=None, session=None):
+    """
+    Retrieve RSE attribute value.
+
+    :param rse_id: The RSE id.
+    :param session: The database session in use.
+
+    :returns: The list of checksums supported by the selected RSE.
+              If the list is empty (aka attribute is not set) it returns all the default checksums.
+              Use 'none' to explicitly tell the RSE does not support any checksum algorithm.
+    """
+
+    checksum_support_attribute_list = get_rse_attribute(key=CHECKSUM_KEY, rse_id=rse_id, session=session)
+
+    if not checksum_support_attribute_list:
+        return GLOBALLY_SUPPORTED_CHECKSUMS
+    else:
+        supported_checksum_list = checksum_support_attribute_list[0].split(',')
+        if 'none' in supported_checksum_list:
+            return []
+        return supported_checksum_list
+
+
+@read_session
+def get_rse_is_checksum_supported(checksum_name, rse_id=None, session=None):
+    """
+    Retrieve RSE attribute value.
+
+    :param checksum_name: The desired checksum name for the attribute.
+    :param rse_id: The RSE id.
+    :param session: The database session in use.
+
+    :returns: True if required checksum is supported, False otherwise.
+    """
+    if is_checksum_valid(checksum_name):
+        return checksum_name in get_rse_supported_checksums(rse_id=rse_id, session=session)
+    else:
+        return False
+
+
 @transactional_session
 def set_rse_usage(rse_id, source, used, free, session=None):
     """
@@ -666,7 +719,7 @@ def delete_rse_limit(rse_id, name=None, session=None):
 
 
 @transactional_session
-def set_rse_transfer_limits(rse_id, activity, rse_expression=None, max_transfers=0, transfers=0, waitings=0, volume=0, session=None):
+def set_rse_transfer_limits(rse_id, activity, rse_expression=None, max_transfers=0, transfers=0, waitings=0, volume=0, deadline=1, strategy='fifo', session=None):
     """
     Set RSE transfer limits.
 
@@ -677,12 +730,16 @@ def set_rse_transfer_limits(rse_id, activity, rse_expression=None, max_transfers
     :param transfers: Current number of tranfers.
     :param waitings: Current number of waitings.
     :param volume: Maximum transfer volume in bytes.
+    :param deadline: Maximum waiting time in hours until a datasets gets released.
+    :param strategy: Stragey to handle datasets `fifo` or `grouped_fifo`.
     :param session: The database session in use.
 
     :returns: True if successful, otherwise false.
     """
     try:
-        rse_tr_limit = models.RSETransferLimit(rse_id=rse_id, activity=activity, rse_expression=rse_expression, max_transfers=max_transfers, transfers=transfers, waitings=waitings, volume=volume)
+        rse_tr_limit = models.RSETransferLimit(rse_id=rse_id, activity=activity, rse_expression=rse_expression,
+                                               max_transfers=max_transfers, transfers=transfers,
+                                               waitings=waitings, volume=volume, strategy=strategy, deadline=deadline)
         rse_tr_limit = session.merge(rse_tr_limit)
         rowcount = rse_tr_limit.save(session=session)
         return rowcount
@@ -714,7 +771,9 @@ def get_rse_transfer_limits(rse_id=None, activity=None, session=None):
             limits[limit.activity][limit.rse_id] = {'max_transfers': limit.max_transfers,
                                                     'transfers': limit.transfers,
                                                     'waitings': limit.waitings,
-                                                    'volume': limit.volume}
+                                                    'volume': limit.volume,
+                                                    'strategy': limit.strategy,
+                                                    'deadline': limit.deadline}
         return limits
     except IntegrityError as error:
         raise exception.RucioException(error.args)
@@ -1132,7 +1191,7 @@ def update_rse(rse_id, parameters, session=None):
     param = {}
     availability_mapping = {'availability_read': 4, 'availability_write': 2, 'availability_delete': 1}
     for key in parameters:
-        if key == 'name' and parameters['name'] != rse:  # Needed due to wrongly setting name in pre1.22.7 clients
+        if key == 'name':
             param['rse'] = parameters['name']
         elif key in ['availability_read', 'availability_write', 'availability_delete']:
             if parameters[key] is True:
@@ -1143,8 +1202,8 @@ def update_rse(rse_id, parameters, session=None):
             param[key] = parameters[key]
     param['availability'] = availability
     query.update(param)
-    if 'rse' in param:
-        add_rse_attribute(rse_id=rse_id, key=parameters['name'], value=True, session=session)
+    if 'name' in parameters:
+        add_rse_attribute(rse_id=rse_id, key=parameters['name'], value=1, session=session)
         query = session.query(models.RSEAttrAssociation).filter_by(rse_id=rse_id).filter(models.RSEAttrAssociation.key == rse)
         rse_attr = query.one()
         rse_attr.delete(session=session)
