@@ -24,6 +24,8 @@
 # - Eric Vaandering <ericvaandering@gmail.com>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Gabriele Fronze' <gfronze@cern.ch>, 2019
+# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019
 #
 # PY3K COMPATIBLE
 
@@ -41,17 +43,19 @@ import traceback
 
 from rucio.common.config import config_get
 from rucio.common.exception import InvalidRSEExpression, TransferToolTimeout, TransferToolWrongAnswer, RequestNotFound, ConfigNotFound, DuplicateFileTransferSubmission
-from rucio.common.utils import chunks
+from rucio.common.utils import chunks, set_checksum_value
 from rucio.core import request, transfer as transfer_core
 from rucio.core.config import get
 from rucio.core.monitor import record_counter, record_timer
-from rucio.core.rse import list_rses
+from rucio.core.rse import list_rses, get_rse_supported_checksums
 from rucio.core.rse_expression_parser import parse_expression
+from rucio.db.sqla.session import read_session
 from rucio.db.sqla.constants import RequestState
 from rucio.rse import rsemanager as rsemgr
 
 USER_ACTIVITY = config_get('conveyor', 'user_activities', False, ['user', 'user_test'])
 USER_TRANSFERS = config_get('conveyor', 'user_transfers', False, None)
+TRANSFER_TOOL = config_get('conveyor', 'transfertool', False, None)
 
 
 def submit_transfer(external_host, job, submitter='submitter', logging_prepend_str='', timeout=None, user_transfer_job=False):
@@ -119,7 +123,7 @@ def submit_transfer(external_host, job, submitter='submitter', logging_prepend_s
         # A eid is returned if the job is properly submitted otherwise an exception is raised
         eid = transfer_core.submit_bulk_transfers(external_host,
                                                   files=job['files'],
-                                                  transfertool='fts3',
+                                                  transfertool=TRANSFER_TOOL,
                                                   job_params=job['job_params'],
                                                   timeout=timeout,
                                                   user_transfer_job=user_transfer_job)
@@ -166,7 +170,7 @@ def submit_transfer(external_host, job, submitter='submitter', logging_prepend_s
                 logging.info('%s About to submit job to %s with timeout %s', prepend_str, external_host, timeout)
                 eid = transfer_core.submit_bulk_transfers(external_host,
                                                           files=[t_file],
-                                                          transfertool='fts3',
+                                                          transfertool=TRANSFER_TOOL,
                                                           job_params=job['job_params'],
                                                           timeout=timeout,
                                                           user_transfer_job=user_transfer_job)
@@ -221,7 +225,8 @@ def submit_transfer(external_host, job, submitter='submitter', logging_prepend_s
         logging.error('%s Failed to submit a job with error %s: %s', prepend_str, str(error), traceback.format_exc())
 
 
-def bulk_group_transfer(transfers, policy='rule', group_bulk=200, source_strategy=None, max_time_in_queue=None):
+@read_session
+def bulk_group_transfer(transfers, policy='rule', group_bulk=200, source_strategy=None, max_time_in_queue=None, session=None):
     """
     Group transfers in bulk based on certain criterias
 
@@ -252,8 +257,34 @@ def bulk_group_transfer(transfers, policy='rule', group_bulk=200, source_strateg
 
     for request_id in transfers:
         transfer = transfers[request_id]
-
         verify_checksum = transfer['file_metadata'].get('verify_checksum', 'both')
+
+        dest_rse_id = transfer['file_metadata']['dest_rse_id']
+        source_rse_id = transfer['file_metadata']['src_rse_id']
+
+        dest_supported_checksums = get_rse_supported_checksums(rse_id=dest_rse_id, session=session)
+        source_supported_checksums = get_rse_supported_checksums(rse_id=source_rse_id, session=session)
+        common_checksum_names = set(source_supported_checksums).intersection(dest_supported_checksums)
+
+        if source_supported_checksums == ['none']:
+            if dest_supported_checksums == ['none']:
+                # both endpoints support none
+                verify_checksum = 'none'
+            else:
+                # src supports none but dst does
+                verify_checksum = 'destination'
+        else:
+            if dest_supported_checksums == ['none']:
+                # source supports some but destination does not
+                verify_checksum = 'source'
+            else:
+                if len(common_checksum_names) == 0:
+                    # source and dst support some bot none in common (dst priority)
+                    verify_checksum = 'destination'
+                else:
+                    # Don't override the value in the file_metadata
+                    pass
+
         t_file = {'sources': transfer['sources'],
                   'destinations': transfer['dest_urls'],
                   'metadata': transfer['file_metadata'],
@@ -263,11 +294,17 @@ def bulk_group_transfer(transfers, policy='rule', group_bulk=200, source_strateg
                   'selection_strategy': source_strategy if source_strategy else activity_source_strategy.get(str(transfer['file_metadata']['activity']), default_source_strategy),
                   'request_type': transfer['file_metadata'].get('request_type', None),
                   'activity': str(transfer['file_metadata']['activity'])}
+
         if verify_checksum != 'none':
-            if 'md5' in list(t_file['metadata'].keys()) and t_file['metadata']['md5']:
-                t_file['checksum'] = 'MD5:%s' % str(t_file['metadata']['md5'])
-            if 'adler32' in list(t_file['metadata'].keys()) and t_file['metadata']['adler32']:
-                t_file['checksum'] = 'ADLER32:%s' % str(t_file['metadata']['adler32'])
+            if verify_checksum == 'both':
+                set_checksum_value(t_file, common_checksum_names)
+            if verify_checksum == 'source':
+                set_checksum_value(t_file, source_supported_checksums)
+            if verify_checksum == 'destination':
+                set_checksum_value(t_file, dest_supported_checksums)
+
+        multihop = transfer.get('multihop', False)
+        strict_copy = transfer.get('strict_copy', False)
 
         external_host = transfer['external_host']
         scope = t_file['metadata']['scope']
@@ -284,13 +321,19 @@ def bulk_group_transfer(transfers, policy='rule', group_bulk=200, source_strateg
                     grouped_transfers[external_host][scope_str] = {}
                     grouped_jobs[external_host][scope_str] = []
 
-        job_params = {'verify_checksum': verify_checksum,
+        job_params = {'account': transfer['account'],
+                      'use_oidc': transfer.get('use_oidc', False),
+                      'verify_checksum': verify_checksum,
                       'copy_pin_lifetime': transfer['copy_pin_lifetime'] if transfer['copy_pin_lifetime'] else -1,
                       'bring_online': transfer['bring_online'] if transfer['bring_online'] else None,
                       'job_metadata': {'issuer': 'rucio'},  # finaly job_meta will like this. currently job_meta will equal file_meta to include request_id and etc.
                       'overwrite': transfer['overwrite'],
                       'priority': 3,
                       's3alternate': True}
+        if multihop:
+            job_params['multihop'] = True
+        if strict_copy:
+            job_params['strict_copy'] = True
 
         # Don't put optional & missing keys in the parameters
         if transfer['dest_spacetoken']:
@@ -321,38 +364,58 @@ def bulk_group_transfer(transfers, policy='rule', group_bulk=200, source_strateg
             if 'max_time_in_queue' in job_params:
                 job_key = job_key + ',%s' % job_params['max_time_in_queue']
 
+            if multihop:
+                job_key = 'multihop_%s' % (transfer['initial_request_id'])
+
             if job_key not in grouped_transfers[external_host]:
                 if USER_TRANSFERS not in ['cms'] or activity not in USER_ACTIVITY:
                     grouped_transfers[external_host][job_key] = {}
                 elif activity in USER_ACTIVITY:
                     grouped_transfers[external_host][scope_str][job_key] = {}
 
-            if policy == 'rule':
-                policy_key = '%s' % (transfer['rule_id'])
-            if policy == 'dest':
-                policy_key = '%s' % (t_file['metadata']['dst_rse'])
-            if policy == 'src_dest':
-                policy_key = '%s,%s' % (t_file['metadata']['src_rse'], t_file['metadata']['dst_rse'])
-            if policy == 'rule_src_dest':
-                policy_key = '%s,%s,%s' % (transfer['rule_id'], t_file['metadata']['src_rse'], t_file['metadata']['dst_rse'])
-            if policy == 'activity_dest':
-                policy_key = '%s %s' % (activity, t_file['metadata']['dst_rse'])
-                policy_key = "_".join(policy_key.split(' '))
-            if policy == 'activity_src_dest':
-                policy_key = '%s %s %s' % (activity, t_file['metadata']['src_rse'], t_file['metadata']['dst_rse'])
-                policy_key = "_".join(policy_key.split(' '))
-            # maybe here we need to hash the key if it's too long
+            if multihop:
+                policy_key = 'multihop_%s' % (transfer['initial_request_id'])
+            else:
+                if policy == 'rule':
+                    policy_key = '%s' % (transfer['rule_id'])
+                if policy == 'dest':
+                    policy_key = '%s' % (t_file['metadata']['dst_rse'])
+                if policy == 'src_dest':
+                    policy_key = '%s,%s' % (t_file['metadata']['src_rse'], t_file['metadata']['dst_rse'])
+                if policy == 'rule_src_dest':
+                    policy_key = '%s,%s,%s' % (transfer['rule_id'], t_file['metadata']['src_rse'], t_file['metadata']['dst_rse'])
+                if policy == 'activity_dest':
+                    policy_key = '%s %s' % (activity, t_file['metadata']['dst_rse'])
+                    policy_key = "_".join(policy_key.split(' '))
+                if policy == 'activity_src_dest':
+                    policy_key = '%s %s %s' % (activity, t_file['metadata']['src_rse'], t_file['metadata']['dst_rse'])
+                    policy_key = "_".join(policy_key.split(' '))
+                    # maybe here we need to hash the key if it's too long
 
             if USER_TRANSFERS not in ['cms'] or activity not in USER_ACTIVITY:
                 if policy_key not in grouped_transfers[external_host][job_key]:
                     grouped_transfers[external_host][job_key][policy_key] = {'files': [t_file], 'job_params': job_params}
                 else:
-                    grouped_transfers[external_host][job_key][policy_key]['files'].append(t_file)
+                    if multihop:
+                        # The parent transfer should be the first of the list
+                        # TODO : Only work for a single hop now, need to be able to handle multiple hops
+                        if transfer['parent_request']:  # This is the child
+                            grouped_transfers[external_host][job_key][policy_key]['files'].append(t_file)
+                        else:
+                            grouped_transfers[external_host][job_key][policy_key]['files'].insert(0, t_file)
+                    else:
+                        grouped_transfers[external_host][job_key][policy_key]['files'].append(t_file)
             elif activity in USER_ACTIVITY:
                 if policy_key not in grouped_transfers[external_host][scope_str][job_key]:
                     grouped_transfers[external_host][scope_str][job_key][policy_key] = {'files': [t_file], 'job_params': job_params}
                 else:
-                    grouped_transfers[external_host][scope_str][job_key][policy_key]['files'].append(t_file)
+                    if multihop:
+                        # The parent transfer should be the first of the list
+                        # TODO : Only work for a single hop now, need to be able to handle multiple hops
+                        if transfer['parent_request']:  # This is the child
+                            grouped_transfers[external_host][scope_str][job_key][policy_key]['files'].append(t_file)
+                        else:
+                            grouped_transfers[external_host][scope_str][job_key][policy_key]['files'].insert(0, t_file)
 
     # for jobs with different job_key, we cannot put in one job.
     for external_host in grouped_transfers:
